#include <imgui.h>
#include <rlImGui.h>
#include <rlgl.h>
#include <stdbool.h>
#ifdef PANDA_INCLUDE_IMGUI

#include "Panda.hpp"
#include <raylib.h>

#if PLATFORM_ANDROID

#include <imgui_impl_android.h>
#include <imgui_impl_opengl3.h>
#include <android_native_app_glue.h>

extern "C" struct android_app *GetAndroidApp(void);

static int32_t handleInputEvent(struct android_app* app, AInputEvent* inputEvent)
{
	return ImGui_ImplAndroid_HandleInputEvent(inputEvent);
}
#endif

void panda::GuiInit()
{
	#if PLATFORM_ANDROID
		GetAndroidApp()->onInputEvent = handleInputEvent;

		IMGUI_CHECKVERSION();
		ImGui::CreateContext();
		ImGuiIO& io = ImGui::GetIO();
		io.IniFilename = NULL;

		ImGui::StyleColorsDark();
		ImGui_ImplAndroid_Init(GetAndroidApp()->window);
		ImGui_ImplOpenGL3_Init("#version 300 es");

		ImFontConfig font_cfg;
		font_cfg.SizePixels = 22.0f;
		io.Fonts->AddFontDefault(&font_cfg);
		ImGui::GetStyle().ScaleAllSizes(3.0f);
	#else
		rlImGuiSetup(true);
	#endif
}

void panda::GuiDestroy()
{
	#if PLATFORM_ANDROID
		ImGui_ImplOpenGL3_Shutdown();
		ImGui_ImplAndroid_Shutdown();
		ImGui::DestroyContext();
	#else
		rlImGuiShutdown();
	#endif
}

void panda::GuiBegin()
{
	#if PLATFORM_ANDROID
		ImGui_ImplOpenGL3_NewFrame();
		ImGui_ImplAndroid_NewFrame();
		ImGui::NewFrame();
	#else
		rlImGuiBegin();
	#endif
}

void panda::GuiEnd()
{
	#if PLATFORM_ANDROID
		rlDrawRenderBatchActive();
		rlDisableBackfaceCulling();

		ImGui::Render();
		ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());

		rlSetTexture(0);
		rlEnableBackfaceCulling();
	#else
		rlImGuiEnd();
	#endif
}

void panda::GuiThemeSetup()
{
	ImGuiStyle& style = ImGui::GetStyle();
	style.AntiAliasedLines = true;
	style.AntiAliasedFill = true;
	style.CurveTessellationTol = 1.25f;
	style.Alpha = 1.f;

	style.FramePadding = ImVec2(4.0f,2.0f);
	style.ItemSpacing = ImVec2(8.0f,2.0f);
	style.WindowRounding = 2.0f;
	style.ChildRounding = 2.0f;
	style.FrameRounding = 0.0f;
	style.ScrollbarRounding = 0.0f;
	style.GrabRounding = 1.0f;
	style.WindowBorderSize = 1.0f;
	style.FrameBorderSize = 1.0f;

	style.WindowRounding = 5.3f;
	style.FrameRounding = 2.3f;
	style.ScrollbarRounding = 0;

	style.Colors[ImGuiCol_Text]                  = ImVec4(0.90f, 0.90f, 0.90f, 0.90f);
	style.Colors[ImGuiCol_TextDisabled]          = ImVec4(0.60f, 0.60f, 0.60f, 1.00f);
	style.Colors[ImGuiCol_WindowBg]              = ImVec4(0.09f, 0.09f, 0.15f, 1.00f);
	style.Colors[ImGuiCol_PopupBg]               = ImVec4(0.05f, 0.05f, 0.10f, 0.85f);
	style.Colors[ImGuiCol_Border]                = ImVec4(0.70f, 0.70f, 0.70f, 0.65f);
	style.Colors[ImGuiCol_BorderShadow]          = ImVec4(0.00f, 0.00f, 0.00f, 0.00f);
	style.Colors[ImGuiCol_FrameBg]               = ImVec4(0.00f, 0.00f, 0.01f, 1.00f);
	style.Colors[ImGuiCol_FrameBgHovered]        = ImVec4(0.90f, 0.80f, 0.80f, 0.40f);
	style.Colors[ImGuiCol_FrameBgActive]         = ImVec4(0.90f, 0.65f, 0.65f, 0.45f);
	style.Colors[ImGuiCol_TitleBg]               = ImVec4(0.00f, 0.00f, 0.00f, 0.83f);
	style.Colors[ImGuiCol_TitleBgCollapsed]      = ImVec4(0.40f, 0.40f, 0.80f, 0.20f);
	style.Colors[ImGuiCol_TitleBgActive]         = ImVec4(0.00f, 0.00f, 0.00f, 0.87f);
	style.Colors[ImGuiCol_MenuBarBg]             = ImVec4(0.01f, 0.01f, 0.02f, 0.80f);
	style.Colors[ImGuiCol_ScrollbarBg]           = ImVec4(0.20f, 0.25f, 0.30f, 0.60f);
	style.Colors[ImGuiCol_ScrollbarGrab]         = ImVec4(0.55f, 0.53f, 0.55f, 0.51f);
	style.Colors[ImGuiCol_ScrollbarGrabHovered]  = ImVec4(0.56f, 0.56f, 0.56f, 1.00f);
	style.Colors[ImGuiCol_ScrollbarGrabActive]   = ImVec4(0.56f, 0.56f, 0.56f, 0.91f);
	style.Colors[ImGuiCol_CheckMark]             = ImVec4(0.90f, 0.90f, 0.90f, 0.83f);
	style.Colors[ImGuiCol_SliderGrab]            = ImVec4(0.70f, 0.70f, 0.70f, 0.62f);
	style.Colors[ImGuiCol_SliderGrabActive]      = ImVec4(0.30f, 0.30f, 0.30f, 0.84f);
	style.Colors[ImGuiCol_Button]                = ImVec4(0.48f, 0.72f, 0.89f, 0.49f);
	style.Colors[ImGuiCol_ButtonHovered]         = ImVec4(0.50f, 0.69f, 0.99f, 0.68f);
	style.Colors[ImGuiCol_ButtonActive]          = ImVec4(0.80f, 0.50f, 0.50f, 1.00f);
	style.Colors[ImGuiCol_Header]                = ImVec4(0.30f, 0.69f, 1.00f, 0.53f);
	style.Colors[ImGuiCol_HeaderHovered]         = ImVec4(0.44f, 0.61f, 0.86f, 1.00f);
	style.Colors[ImGuiCol_HeaderActive]          = ImVec4(0.38f, 0.62f, 0.83f, 1.00f);
	style.Colors[ImGuiCol_Separator]                = ImVec4(0.50f, 0.50f, 0.50f, 1.00f);
	style.Colors[ImGuiCol_SeparatorHovered]         = ImVec4(0.70f, 0.60f, 0.60f, 1.00f);
	style.Colors[ImGuiCol_SeparatorActive]          = ImVec4(0.90f, 0.70f, 0.70f, 1.00f);
	style.Colors[ImGuiCol_ResizeGrip]            = ImVec4(1.00f, 1.00f, 1.00f, 0.85f);
	style.Colors[ImGuiCol_ResizeGripHovered]     = ImVec4(1.00f, 1.00f, 1.00f, 0.60f);
	style.Colors[ImGuiCol_ResizeGripActive]      = ImVec4(1.00f, 1.00f, 1.00f, 0.90f);
	style.Colors[ImGuiCol_PlotLines]             = ImVec4(1.00f, 1.00f, 1.00f, 1.00f);
	style.Colors[ImGuiCol_PlotLinesHovered]      = ImVec4(0.90f, 0.70f, 0.00f, 1.00f);
	style.Colors[ImGuiCol_PlotHistogram]         = ImVec4(0.90f, 0.70f, 0.00f, 1.00f);
	style.Colors[ImGuiCol_PlotHistogramHovered]  = ImVec4(1.00f, 0.60f, 0.00f, 1.00f);
	style.Colors[ImGuiCol_TextSelectedBg]        = ImVec4(0.00f, 0.00f, 1.00f, 0.35f);
	style.Colors[ImGuiCol_ModalWindowDimBg]  = ImVec4(0.20f, 0.20f, 0.20f, 0.35f);
}

#endif


/////////////////////////////////




/////////////////////////////////

// panda::TilemapMetadata panda::LoadTilemapMetadata(const char *file)
// {
//     ini::IniFile ini;
//     char* text = LoadFileText(file);
//     ini.decode(text);

//     return TilemapMetadata{
//         ini["chunk"]["width"].as<int>(),
//         ini["chunk"]["height"].as<int>(),
//         ini["tile"]["width"].as<int>(),
//         ini["tile"]["height"].as<int>(),
//     };
// }

// panda::TilemapItem panda::LoadTilemap(const char *file, panda::TilemapMetadata metadata, Texture texture)
// {
//     return TilemapItem();
// }
