#include "Panda.hpp"
#include <raylib.h>

void panda::DrawGridMap(Texture texture, Camera2D cam)
{
  Vector2 topLeft = GetScreenToWorld2D({0.0f, 0.0f}, cam);
  Vector2 botRight = GetScreenToWorld2D({(float)GetScreenWidth(), (float)GetScreenHeight()}, cam);
  float offsetX = (int)topLeft.x % texture.width;
  float offsetY = (int)topLeft.y % texture.width;

  DrawTextureTiled(
    texture,
    {offsetX, offsetY, (float)texture.width, (float)texture.height},
    {topLeft.x, topLeft.y, botRight.x-topLeft.x, botRight.y-topLeft.y},
    {0.0f, 0.0f}, 0.0f, 1.0f, WHITE
  );
}

void panda::DrawGridMapQuick(Texture texture, Vector2 offset)
{
  float offsetX = (int)offset.x % texture.width;
  float offsetY = (int)offset.y % texture.width;

  DrawTextureTiled(
    texture,
    {offsetX, offsetY, (float)texture.width, (float)texture.height},
    {offset.x, offset.y, (float)GetScreenWidth(), (float)GetScreenHeight()},
    {0.0f, 0.0f}, 0.0f, 1.0f, WHITE
  );
}

Texture panda::LoadDefaultGridMap()
{
  Image image = GenImageColor(100, 100, WHITE);
  ImageDrawLine(&image, image.width*0.5f, 0, image.width*0.5f, image.height, {0, 0, 0, 120});
  ImageDrawLine(&image, 0, image.height*0.5f, 100, image.height*0.5f, {0, 0, 0, 120});
  ImageDrawRectangleLines(&image, {0.0f, 0.0f, (float)image.width, (float)image.height}, 1, {0, 0, 0, 180});

  Texture tex = LoadTextureFromImage(image);
  UnloadImage(image);

  return tex;
}
