#include "Panda.hpp"
#include <cmath>

namespace panda {
  void Matrix::identity()
  {
    a = 1.0f;
    b = 0.0f;
    c = 0.0f;
    d = 1.0f;
    tx = 0.0f;
    ty = 0.0f;
  }

  void Matrix::translate(float dx, float dy)
  {
    Matrix m;
		m.tx = dx;
		m.ty = dy;
		concat(m);
  }

  void Matrix::rotate(float radian)
  {
    float cos = std::cos(radian);
		float sin = std::sin(radian);
		
		float a1 = a * cos - b * sin;
		b = a * sin + b * cos;
		a = a1;
		
		float c1 = c * cos - d * sin;
		d = c * sin + d * cos;
		c = c1;
		
		float tx1 = tx * cos - ty * sin;
		ty = tx * sin + ty * cos;
		tx = tx1;
  }

  void Matrix::scale(float sx, float sy)
  {
    a *= sx;
		b *= sy;
		c *= sx;
		d *= sy;
		tx *= sx;
		ty *= sy;
  }

  void Matrix::concat(Matrix m)
  {
    float a1 = a * m.a + b * m.c;
		b = a * m.b + b * m.d;
		a = a1;

		float c1 = c * m.a + d * m.c;
		d = c * m.b + d * m.d;
		c = c1;
		
		float tx1 = tx * m.a + ty * m.c + m.tx;
		ty = tx * m.b + ty * m.d + m.ty;
		tx = tx1;
  }
}