#pragma once
#include <cstddef>
#include <list>

namespace panda
{
  template<typename T, size_t SIZE>
  class PoolPointer {
   private:
    std::list<T*> free;

   public:
    PoolPointer() : free()
    {
      for (size_t i=0; i<SIZE; i++) free.emplace_back(new T());
    }

    T* get() {
      if (free.empty())
        return new T(); // no objects are free
      
      // get the last object in the list
      auto obj = free.back();
      free.pop_back();
      return obj;
    }

    int getFreeSize()
    {
      return free.size();
    }

    /// Mark the given object for reuse in the future.
    inline void put(T* obj) { free.emplace_back(obj); }
  };

  template<typename T, size_t SIZE>
  class PoolCircular {
    public:
      T pool[SIZE];

      PoolCircular() :
        head(0)
      { }

      T *Get()
      {
        head = (head + 1) % SIZE;
        return pool[head];
      }

      size_t size() { return SIZE; }
      int getHead() { return head; } // for debug only!!!
    
    private:
      int head;
  };
}