#include "Panda.hpp"

static inline uint64_t splitmix64(uint64_t &x) {
  uint64_t z = (x += 0x9e3779b97f4a7c15);
  z = (z ^ (z >> 30)) * 0xbf58476d1ce4e5b9;
  z = (z ^ (z >> 27)) * 0x94d049bb133111eb;
  return z ^ (z >> 31);
}

static inline uint64_t rotl(const uint64_t x, int k) {
  return (x << k) | (x >> (64 - k));
}

panda::Random::Random(uint64_t seed) {
  for(int i=0; i<4; i++) s[i] = splitmix64(seed);
}

void panda::Random::SetSeed(uint64_t seed) {
  for(int i=0; i<4; i++) s[i] = splitmix64(seed);
}

uint64_t panda::Random::Next() {
  const uint64_t result = s[0] + s[3];

  const uint64_t t = s[1] << 17;

  s[2] ^= s[0];
  s[3] ^= s[1];
  s[1] ^= s[2];
  s[0] ^= s[3];

  s[2] ^= t;

  s[3] = rotl(s[3], 45);

  return result;
}

double panda::Random::Next_UNI() {
  return (Next() >> 11) * 0x1.0p-53;
}

double panda::Random::Range(double min, double max) {
  return min + (max-min) * Next_UNI();
}