#include "Panda.hpp"
#include <raylib.h>

void panda::DrawSprite(Texture &texture, const panda::Sprite &sprite, const int &x, const int &y, const Color &color)
{
  DrawSpriteEx(texture, sprite, {(float)x, (float)y}, {1.0f, 1.0f}, 0.0f, false, false, color);
}

void panda::DrawSpriteV(Texture &texture, const panda::Sprite &sprite, const Vector2 &pos, const Color &color)
{
  DrawSpriteEx(texture, sprite, pos, {1.0f, 1.0f}, 0.0f, false, false, color);
}

void panda::DrawSpriteEx(Texture &texture, const panda::Sprite &sprite, const Vector2 &pos, const Vector2 &scale, const float &rotation, bool flipX, bool flipY, const Color &color)
{
  panda::Matrix m;

  if (sprite.isRotated)
    m.translate(0, sprite.height);
  
  m.translate(
    ((flipX) ? -1+sprite.pivotX : -sprite.pivotX) *sprite.width,
    ((flipY) ? -1+sprite.pivotY : -sprite.pivotY) *sprite.height);

  m.translate(
    ((flipX) ? -1 : 1) * sprite.offsetX,
    ((flipY) ? -1 : 1) * sprite.offsetY);

  m.scale(scale.x, scale.y);
  m.rotate(rotation * DEG2RAD);
  m.translate(pos.x, pos.y);
  
  DrawTexturePro(
    texture,
    {
      (float)sprite.frameX, (float)sprite.frameY,
      ((flipY) ? -1.0f : 1.0f) * sprite.frameW,
      ((flipY) ? -1.0f : 1.0f) * sprite.frameH
    },
    {
      m.tx, m.ty,
      (sprite.isRotated) ? sprite.height*scale.y : sprite.width*scale.x,
      (sprite.isRotated) ? sprite.width*scale.x : sprite.height*scale.y
    },
    {0.0f, 0.0f},
    (sprite.isRotated) ? rotation-90 : rotation,
    color);
}

void panda::DrawTextureMap(const panda::TextureMap &atlas, const char* texture, const int &x, const int &y, const Color &color)
{
  DrawSpriteEx(*atlas.texture, atlas.map.at(texture), {(float)x, (float)y}, {1.0f, 1.0f}, 0.0f, false, false, color);
}

void panda::DrawTextureMapV(const panda::TextureMap &atlas, const char* texture, const Vector2 &pos, const Color &color)
{

}

void panda::DrawTextureMapEx(const panda::TextureMap &atlas, const char* texture, const Vector2 &pos, const Vector2 &scale, const float &rotation, bool flipX, bool flipY, const Color &color)
{

}
